#include <SDL/SDL.h>
#include "gameengine.h"
#include "gamestate.h"
#include "main.h"
void CGameEngine::Init(const char* title, int width, int height,
					   int bpp, bool fullscreen)
{
	int flags = 0;


	if (SDL_Init(SDL_INIT_EVERYTHING) < 0)
	{
		fprintf (stderr, "Couldn't initialize SDL: %s\n", SDL_GetError ());
		exit (1);
	}
	atexit (Terminate);
	if ( fullscreen )
	{
		flags = SDL_FULLSCREEN;
	}
	flags=flags|SDL_SWSURFACE;
	screen = SDL_SetVideoMode(width, height, bpp, flags);
	if (screen == NULL)
	{
		fprintf (stderr, "Couldn't set 320x240x16 video mode: %s\n", SDL_GetError ());
		exit (2);
	}
	if (SDL_NumJoysticks() > 0)
	{
		joy = SDL_JoystickOpen(0);
		if(!joy)
		{
			fprintf (stderr, "Couldn't open joystick 0: %s\n", SDL_GetError ());
		}
	}
	m_fullscreen = fullscreen;
	m_running = true;

}

void CGameEngine::Cleanup()
{
	while ( !states.empty() )
	{
		states.back()->Cleanup();
		states.pop_back();
	}

	// switch back to windowed mode so other
	// programs won't get accidentally resized
	if ( m_fullscreen )
	{
		screen = SDL_SetVideoMode(640, 480, 0, 0);
	}

	SDL_Quit();
}

void CGameEngine::ChangeState(CGameState* state)
{
	// cleanup the current state
	if ( !states.empty() )
	{
		states.back()->Cleanup();
		states.pop_back();
	}

	// store and init the new state
	states.push_back(state);
	states.back()->Init();
}

void CGameEngine::PushState(CGameState* state)
{
	// pause current state
	if ( !states.empty() )
	{
		states.back()->Pause();
	}

	// store and init the new state
	states.push_back(state);
	states.back()->Init();
}

void CGameEngine::PopState()
{
	// cleanup the current state
	if ( !states.empty() )
	{
		states.back()->Cleanup();
		states.pop_back();
	}

	// resume previous state
	if ( !states.empty() )
	{
		states.back()->Resume();
	}
}

void CGameEngine::HandleEvents()
{
	// let the state handle events
	states.back()->HandleEvents(this);
}

void CGameEngine::Update()
{
	// let the state update the game
	states.back()->Update(this);
}

void CGameEngine::Draw()
{
	// let the state draw the screen
	states.back()->Draw(this);
}
