#include "graphics.h"
void ClearScr(SDL_Surface *screen, Uint8 R, Uint8 G, Uint8 B)
{
	int x1=0;
	int y1=0;
	int x2=screen->w;
	int y2=screen->h;

	Uint32 color = SDL_MapRGB(screen->format, R, G, B);
	switch (screen->format->BytesPerPixel)
	{
	case 1:
		{
			Uint8 *bufp;
			for(int x=x1; x<x2*y2; x++)
			{
				for(int y=y1; y<y2; y++)
				{
					bufp = (Uint8 *)screen->pixels + y*screen->pitch + x;
					*bufp = color;
				}
			}
		}
		break;

	case 2:
		{
			Uint16 *bufp;
			for(int x=x1; x<x2; x++)
			{
				for(int y=y1; y<y2; y++)
				{
					bufp = (Uint16 *)screen->pixels + y*screen->pitch/2 + x;
					*bufp = color; 
				}
			}
		}
		break;

	case 3:
		{
			Uint8 *bufp;
			for(int x=x1; x<x2; x++)
			{
				for(int y=y1; y<y2; y++)
				{
					bufp = (Uint8 *)screen->pixels + y*screen->pitch + x * 3;
					if(SDL_BYTEORDER == SDL_LIL_ENDIAN) 
					{
						bufp[0] = color;
						bufp[1] = color >> 8;
						bufp[2] = color >> 16;
					} 
					else 
					{
						bufp[2] = color;
						bufp[1] = color >> 8;
						bufp[0] = color >> 16;
					}
				}
			}

		}
		break;

	case 4:
		{
			Uint32 *bufp;
			for(int x=x1; x<x2; x++)
			{
				for(int y=y1; y<y2; y++)
				{
					bufp = (Uint32 *)screen->pixels + y*screen->pitch/4 + x;
					*bufp = color;    
				}
			}
		}
		break;
	}
}

void DrawPixel(SDL_Surface *screen, int x, int y, Uint8 R, Uint8 G, Uint8 B)
{
	Uint32 color = SDL_MapRGB(screen->format, R, G, B);

	if ( SDL_MUSTLOCK(screen) )
	{
		if ( SDL_LockSurface(screen) < 0 )
		{
			return;
		}

	}
	switch (screen->format->BytesPerPixel)
	{
	case 1:
		{
			Uint8 *bufp;

			bufp = (Uint8 *)screen->pixels + y*screen->pitch + x;
			*bufp = color;
		}
		break;

	case 2:
		{
			Uint16 *bufp;

			bufp = (Uint16 *)screen->pixels + y*screen->pitch/2 + x;
			*bufp = color;
		}
		break;

	case 3:
		{
			Uint8 *bufp;

			bufp = (Uint8 *)screen->pixels + y*screen->pitch + x * 3;
			if(SDL_BYTEORDER == SDL_LIL_ENDIAN)
			{
				bufp[0] = color;
				bufp[1] = color >> 8;
				bufp[2] = color >> 16;
			}
			else
			{
				bufp[2] = color;
				bufp[1] = color >> 8;
				bufp[0] = color >> 16;
			}
		}
		break;

	case 4:
		{
			Uint32 *bufp;

			bufp = (Uint32 *)screen->pixels + y*screen->pitch/4 + x;
			*bufp = color;
		}
		break;
	}
	if ( SDL_MUSTLOCK(screen) )
	{
		SDL_UnlockSurface(screen);
	}
}


void DrawFillRect(SDL_Surface *screen, int x1, int y1, int x2, int y2, Uint8 R, Uint8 G, Uint8 B)
{
	ClearScr(screen);
	Uint32 color = SDL_MapRGB(screen->format, R, G, B);

	if ( SDL_MUSTLOCK(screen) )
	{
		if ( SDL_LockSurface(screen) < 0 )
		{
			return;
		}
	}
	switch (screen->format->BytesPerPixel)
	{
	case 1:
		{
			Uint8 *bufp;
			for(int x=x1; x<x2; x++)
			{
				for(int y=y1; y<y2; y++)
				{
					bufp = (Uint8 *)screen->pixels + y*screen->pitch + x;
					*bufp = color;
				}
			}
		}
		break;

	case 2:
		{
			Uint16 *bufp;
			for(int x=x1; x<x2; x++)
			{
				for(int y=y1; y<y2; y++)
				{
					bufp = (Uint16 *)screen->pixels + y*screen->pitch/2 + x;
					*bufp = color; 
				}
			}
		}
		break;

	case 3:
		{
			Uint8 *bufp;
			for(int x=x1; x<x2; x++)
			{
				for(int y=y1; y<y2; y++)
				{
					bufp = (Uint8 *)screen->pixels + y*screen->pitch + x * 3;
					if(SDL_BYTEORDER == SDL_LIL_ENDIAN) 
					{
						bufp[0] = color;
						bufp[1] = color >> 8;
						bufp[2] = color >> 16;
					} 
					else 
					{
						bufp[2] = color;
						bufp[1] = color >> 8;
						bufp[0] = color >> 16;
					}
				}
			}

		}
		break;

	case 4:
		{
			Uint32 *bufp;
			for(int x=x1; x<x2; x++)
			{
				for(int y=y1; y<y2; y++)
				{
					bufp = (Uint32 *)screen->pixels + y*screen->pitch/4 + x;
					*bufp = color;    
				}
			}
		}
		break;
	}
	if ( SDL_MUSTLOCK(screen) )
	{
		SDL_UnlockSurface(screen);
	}
}

void DrawLine(SDL_Surface *screen, int x1, int y1, int x2, int y2, Uint8 R, Uint8 G, Uint8 B)
{
	Uint32 color = SDL_MapRGB(screen->format, R, G, B);
	int px=0;
	int py=0;
	if ( SDL_MUSTLOCK(screen) )
	{
		if ( SDL_LockSurface(screen) < 0 )
		{
			return;
		}
	}
	switch (screen->format->BytesPerPixel)
	{
	case 1:
		{
			Uint8 *bufp;
			for(double degree=0; degree<1; degree+=0.001f)
			{
				px = (int)(x1 + (x2-x1) * degree);
				py = (int)(y1 + (y2-y1) * degree);
				bufp = (Uint8 *)screen->pixels + py*screen->pitch + px;
				*bufp = color;
			}
		}
		break;

	case 2:
		{
			Uint16 *bufp;
			for(double degree=0; degree<1; degree+=0.001f)
			{
				px = (int)(x1 + (x2-x1) * degree);
				py = (int)(y1 + (y2-y1) * degree);
				bufp = (Uint16 *)screen->pixels + py*screen->pitch/2 + px;
				*bufp = color; 
			}
		}
		break;

	case 3:
		{
			Uint8 *bufp;
			for(double degree=0; degree<1; degree+=0.001f)
			{
				px = (int)(x1 + (x2-x1) * degree);
				py = (int)(y1 + (y2-y1) * degree);
				bufp = (Uint8 *)screen->pixels + py*screen->pitch + px * 3;
				if(SDL_BYTEORDER == SDL_LIL_ENDIAN) 
				{
					bufp[0] = color;
					bufp[1] = color >> 8;
					bufp[2] = color >> 16;
				} 
				else 
				{
					bufp[2] = color;
					bufp[1] = color >> 8;
					bufp[0] = color >> 16;
				}
			}

		}
		break;

	case 4:
		{
			Uint32 *bufp;
			for(double degree=0; degree<1; degree+=0.001f)
			{
				px = (int)(x1 + (x2-x1) * degree);
				py = (int)(y1 + (y2-y1) * degree);
				bufp = (Uint32 *)screen->pixels + py*screen->pitch/4 + px;
				*bufp = color;    
			}
		}
		break;
	}
	if ( SDL_MUSTLOCK(screen) )
	{
		SDL_UnlockSurface(screen);
	}
}

void DrawCircle(SDL_Surface *screen, int x, int y, int radius, Uint8 R, Uint8 G, Uint8 B)
{	
	ClearScr(screen);
	Uint32 color = SDL_MapRGB(screen->format, R, G, B);
	int px=0;
	int py=0;
	if ( SDL_MUSTLOCK(screen) )
	{
		if ( SDL_LockSurface(screen) < 0 )
		{
			return;
		}
	}
	switch (screen->format->BytesPerPixel)
	{
	case 1:
		{
			Uint8 *bufp;
			for(double degree=0; degree<2*3.131592; degree+=0.005f)
			{
				px = (int)(x + cos(degree) * radius);
				py = (int)(y - sin(degree) * radius);
				if(screen->w>px&&screen->h>py&&0<px&&0<py)
				{
					bufp = (Uint8 *)screen->pixels + py*screen->pitch + px-1;
					*bufp = color;
				}
			}
		}
		break;

	case 2:
		{
			Uint16 *bufp;
			for(double degree=0; degree<2*3.131592; degree+=0.005f)
			{
				px = (int)(x + cos(degree) * radius);
				py = (int)(y - sin(degree) * radius);
				if(screen->w>px&&screen->h>py&&0<px&&0<py)
				{
					bufp = (Uint16 *)screen->pixels + py*screen->pitch/2 + px-2;
					*bufp = color; 
				}
			}
		}
		break;

	case 3:
		{
			Uint8 *bufp;
			for(double degree=0; degree<2*3.131592; degree+=0.005f)
			{
				px = (int)(x + cos(degree) * radius);
				py = (int)(y - sin(degree) * radius);
				if(screen->w>px&&screen->h>py&&0<px&&0<py)
				{
					bufp = (Uint8 *)screen->pixels + py*screen->pitch + px * 3-3;
					if(SDL_BYTEORDER == SDL_LIL_ENDIAN) 
					{
						bufp[0] = color;
						bufp[1] = color >> 8;
						bufp[2] = color >> 16;
					} 
					else 
					{
						bufp[2] = color;
						bufp[1] = color >> 8;
						bufp[0] = color >> 16;
					}
				}
			}

		}
		break;

	case 4:
		{
			Uint32 *bufp;
			for(double degree=0; degree<2*3.131592; degree+=0.005f)
			{
				px = (int)(x + cos(degree) * radius);
				py = (int)(y - sin(degree) * radius);
				if(screen->w>px&&screen->h>py&&0<px&&0<py)
				{
					bufp = (Uint32 *)screen->pixels + py*screen->pitch/4 + px-4;
					*bufp = color;   
				}
			}
		}
		break;
	}
	if ( SDL_MUSTLOCK(screen) )
	{
		SDL_UnlockSurface(screen);
	}

	SDL_UpdateRect(screen, x-radius, y-radius, x+radius, y+radius);
}
